from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5.uic import loadUi
from PyQt5.QtGui import QPixmap
import sys
import random
import networkx as nx
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from collections import deque


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        loadUi('ui.ui', self)
        self.pushButton.clicked.connect(self.final_generation)
        self.radioButton.toggled.connect(self.on_radio_button_toggled)
        self.radioButton_2.toggled.connect(self.on_radio_button_toggled)
        self.radioButton.setChecked(True)
        self.pushButton_2.clicked.connect(self.show_answer)
        self.pushButton_2.hide()
        self.label_5.setVisible(False)
        self.show()

    # Проверяем, какой тип задания выбрал пользователь, если нет, то 1

    type_of_task = 1
    def on_radio_button_toggled(self):
        global type_of_task
        sender = self.sender()
        if sender.isChecked():
            if sender == self.radioButton:
                type_of_task = 1
            elif sender == self.radioButton_2:
                type_of_task = 2

    def show_answer(self):
        self.label_5.setVisible(True)

    def final_generation(self):
        global answer_for_type2
        # Генерация матрицы размером size со случайным заполнением 0 и 1
        def matrix_generation(size):
            array = [[0 for _ in range(size)] for _ in range(size)]
            for i in range(size):
                for j in range(i + 1, size):
                    array[i][j] = random.randint(0, 1)
                    array[j][i] = array[i][j]
            return array

        # Проверка матрицы на требование: не менее 2 и не более 4 "1" в каждой строке матрицы
        def matrix_check(array):
            for row in array:
                count = sum(row)
                if not (count >= 2 and count <= 3):
                    return False
            return True

        # Финальное создание задания и сохранение графа
        def saving_matrix(resultmatrix):
            # G - граф
            G = nx.Graph()

            # Случайно перемешиваем буквы из списка letters
            letters = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
            random.shuffle(letters)
            nums = []
            for i in range(len(resultmatrix)):
                G.add_node(i)
                nums.append(i)

            # Создаем словарь вида Номер точки на графе: Буквенное обозначение
            answer = dict(zip(nums, letters))

            # Случайно выбираем буквы, номер которых нужно будет найти пользователю. Генерация происходит до тех пор, пока буквы будут различными
            required_letter_1 = random.choice(letters)
            required_letter_2 = random.choice(letters)
            while not(required_letter_1 != required_letter_2):
                required_letter_2 = random.choice(letters)

            # Создаем словарь вида Буквенное обозначение: Номер точки на графе
            answer_2 = dict(zip(letters, nums))

            # Задаем ребра графа
            for i in range(len(resultmatrix)):
                for j in range(i + 1, len(resultmatrix)):
                    if resultmatrix[i][j] == 1:
                        G.add_edge(i, j)

            # Создание нового графика с помощью библиотеки Matplotlib. fig представляет собой объект Figure, а ax - объект Axes, который представляет область построения на графике.
            fig, ax = plt.subplots()

            # Вычисление позиции узлов графа G алгоритмом spring_layout
            pos = nx.spring_layout(G)

            # Рисуем граф
            nx.draw(G, pos, with_labels=True, font_weight='bold', font_size=10, labels=answer)

            # Создаем фигуру canvas с помощью FigureCanvasQTAgg
            canvas = FigureCanvas(fig)

            # Присваимваем label_3 фигуру canvas
            self.label_3.setPixmap(QPixmap(canvas.grab()))
            self.label_3.setVisible(True)
            answer_text = "Ответы:\n"
            for node, letter in answer.items():
                answer_text += f"П{node+1}: {letter}, "
            self.label_2.setText(f'На рисунке справа схема дорог Н-ского района изображена в виде графа, звёздочка в ячейке таблицы обозначает наличие дороги между двумя пунктами. Так как таблицу и схему рисовали независимо друг от друга, то нумерация населённых пунктов в таблице никак не связана с буквенными обозначениями на графе. Определите, какие номера населённых пунктов в таблице могут соответствовать населённым пунктам {required_letter_1} и {required_letter_2} на схеме. В качестве ответа перечислите найденные номера в любом порядке.')
            self.label_5.setText(f'Ответ: {answer_2[required_letter_1]+1}, {answer_2[required_letter_2]+1}')
            self.pushButton_2.show()
            self.label_5.setVisible(False)
            return answer


        resulting_matrix = matrix_generation(8)
        while not matrix_check(resulting_matrix):
            resulting_matrix = matrix_generation(8)


        saving_matrix(resulting_matrix)

        if type_of_task == 1:
            for i in range(8):
                for j in range(8):
                    if resulting_matrix[i][j] == 1:
                        resulting_matrix[i][j] = '*'
                    else:
                        resulting_matrix[i][j] = '-'
        else:
            answer = saving_matrix(resulting_matrix)
            required_number_1 = random.randint(0, 7)
            required_number_2 = random.randint(0, 7)
            while not (required_number_1 != required_number_2):
                required_number_2 = random.randint(0, 7)
            adj_list = {}
            for i in range(8):
                for j in range(8):
                    if resulting_matrix[i][j] == 1:
                        resulting_matrix[i][j] = random.randint(1, 9)
                    else:
                        resulting_matrix[i][j] = '-'

            for i in range(len(resulting_matrix)):
                for j in range(len(resulting_matrix[i])):
                    if resulting_matrix[i][j] != '-':
                        if i not in adj_list:
                            adj_list[i] = []
                        adj_list[i].append((j, int(resulting_matrix[i][j])))

            def bfs(adj_list, start, end):
                visited = set()
                queue = deque([(start, 0)])  # Добавляем начальную вершину и длину пути
                visited.add(start)
                while queue:
                    node, distance = queue.popleft()
                    if node == end:
                        return distance
                    if node in adj_list:
                        for neighbor, edge_distance in adj_list[node]:
                            if neighbor not in visited:
                                queue.append((neighbor, distance + edge_distance))
                                visited.add(neighbor)
            path_length = bfs(adj_list, required_number_1, required_number_2)
            self.label_2.setText(
                f'На рисунке справа схема дорог Н-ского района изображена в виде графа, звёздочка в ячейке таблицы обозначает наличие дороги между двумя пунктами. Так как таблицу и схему рисовали независимо друг от друга, то нумерация населённых пунктов в таблице никак не связана с буквенными обозначениями на графе. Определите протяженность пути между населённым пунктам {answer[required_number_1]} и {answer[required_number_2]} на схеме. В ответе запишите целое число.')
            self.label_5.setText(f'Ответ: {path_length}')

        result_text = "     П1: П2: П3: П4: П5: П6: П7: П8:    \n"
        for i, row in enumerate(resulting_matrix, 1):
            result_row = ''
            for j in row:
                result_row += (' '+ str(j) + '    ')
            result_text += f"П{i}: {result_row}\n"

        self.label_4.setText(result_text)
        self.label_2.setVisible(True)
        self.label.setVisible(False)
        self.pushButton.setText('Далее')
        self.pushButton.move(10, 466)
        self.radioButton.move(130, 465)
        self.radioButton_2.move(130, 485)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    app.exec_()

